/**
  *  \file client/actions/gamission.cc
  *  \brief Global Actions: Missions
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/actions/gamission.h"
#include "util/strlist.h"
#include "ui/listbox.h"
#include "game/racename.h"
#include "ui/msgbox.h"
#include "cpluslib/format.h"
#include "util/nls.h"
#include "game/ship.h"
#include "game/planet.h"
#include "game/tables.h"
#include "game/mission.h"
#include "int/bytecode.h"
#include "int/simple.h"
#include "client/dlg-mission.h"
#include "game/team.h"
#include "game/reg.h"
#include "int/exec.h"
#include "game/config.h"
#include "int/expr.h"
#include "int/process.h"

/*************************** Starbase Missions ***************************/

class WGlobalBaseMissionAction::Executer : public WGlobalActionExecuter {
    int mission;
 public:
    Executer(int mission)
        : mission(mission)
        { }
    bool execute(GObject& obj);
    void finish(int obj, int success);
};

WGlobalActionExecuter*
WGlobalBaseMissionAction::prepare()
{
    UIStringList data;
    for (int i = 0; i <= 6; ++i)
        data.add(i, format("%d - %s", i, getBaseMissionName(i)));

    UIStandardListbox lb(0, 7, 300, data);
    if (lb.doStandardListbox(_("Starbase Mission"), 0 /* FIXME: help id */)) {
        return new Executer(lb.getKey());
    } else {
        return 0;
    }
}

bool
WGlobalBaseMissionAction::Executer::execute(GObject& obj)
{
    if (GPlanet* pl = dynamic_cast<GPlanet*>(&obj)) {
        if (pl->isPlayed() && pl->hasBase()) {
            if (pl->getBaseMission() != mission) {
                pl->setBaseMission(mission);
                return true;
            }
        }
    }
    return false;
}

void
WGlobalBaseMissionAction::Executer::finish(int /*obj*/, int success)
{
    messageBox(format(_("Mission changed on %d starbase%!1{s%}."), success), _("Global Actions"));
}

/*************************** Starship Missions ***************************/

class WGlobalShipMissionAction::Executer : public WGlobalActionExecuter {
    TMissionData mission;
 public:
    Executer(TMissionData mission)
        : mission(mission)
        { }
    bool execute(GObject& obj);
    void finish(int obj, int success);
};

static bool
checkMission(const GMission& msn)
{
    /* We cannot do all missions from the Global Actions. We exclude:
       - missions of other players (even if we have RCd ships of them)
       - missions that are registered-only
       - missions that require waypoint permissions
       - missions whose args require the notion of "here"
       - missions whose condition evaluates to FALSE without error
         (i.e. do not depend on actual ship data)
       Note that the last item is actually implemented as "evaluates to TRUE"
       in PCC 1.x, although documented like here */
    if (!msn.getRaceMask().contains(raceId(::getPlayerId())))
        return false;
    if ((msn.getFlags() & GMission::mf_Registered) != 0 && !allowRegistered(::getPlayerId()))
        return false;
    if ((msn.getFlags() & GMission::mf_Waypoint) != 0)
        return false;
    if ((msn.getArg(GMission::InterceptParameter) & GMission::ma_TypeMask) == GMission::mat_SID_here)
        return false;
    if ((msn.getArg(GMission::TowParameter) & GMission::ma_TypeMask) == GMission::mat_SID_here)
        return false;

    IntBCORef bco = compileExpression(msn.getCondition(), IntCompilationContext());
    if (bco) {
        // Run it
        IntExecutionContext exec("Temporary: Mission");
        exec.pushNewFrame(new IntExecutionFrame(bco));
        if (runTemporaryProcess(exec)) {
            // Evaluated successfully
            if (getBoolValue(exec.getResult()) == 0) {
                return false;
            }
        }
    }

    /* We cannot prove it's disallowed, so allow it */
    return true;
}

WGlobalActionExecuter*
WGlobalShipMissionAction::prepare()
{
    GMissionList list;

    /* Generate list of missions that can be done from global actions. */
    const GMissionList& all = GMissionList::getInstance();
    for (GMissionList::iterator i = all.begin(); i != all.end(); ++i) {
        if (checkMission(*i)) {
            list.addMission(*i);
        }
    }

    /* Ask for parameters */
    TMissionData msn;
    msn.mission = 0;
    msn.args[0] = msn.args[1] = 0;
    if (chooseMission(list, 0, msn)) {
        return new Executer(msn);
    } else {
        return 0;
    }
}

bool
WGlobalShipMissionAction::Executer::execute(GObject& obj)
{
    if (GShip* sh = dynamic_cast<GShip*>(&obj)) {
        if (sh->isPlayed()) {
            // FIXME: PCC 1.x also checks that we do not overwrite a locked mission with
            // waypoint permissions, i.e. a script that has set Intercept.
            if (mission.mission != sh->getMission()
                || mission.args[0] != sh->getIntId()
                || mission.args[1] != sh->getTowId())
            {
                sh->setMission(mission.mission, mission.args[0], mission.args[1]);
                return true;
            }
        }
    }
    return false;
}

void
WGlobalShipMissionAction::Executer::finish(int /*obj*/, int success)
{
    messageBox(format(_("Mission changed on %d ship%!1{s%}."), success), _("Global Actions"));
}

/***************************** Starship enemy ****************************/

class WGlobalShipEnemyAction::Executer : public WGlobalActionExecuter  {
    int enemy;
 public:
    Executer(int enemy)
        : enemy(enemy)
        { }
    bool execute(GObject& obj);
    void finish(int obj, int success);
};

WGlobalActionExecuter*
WGlobalShipEnemyAction::prepare()
{
    StringList d;
    d.add(0, _("0 - none"));
    for (int i = 1; i <= NUM_PLAYERS; ++i)
        d.add(i, format("%X - %s", i, player_racenames.getShortName(i)));

    UIStandardListbox lb(0, -1, 240, d);
    if (lb.doStandardListbox(_("Change Primary Enemy"), 0)) {
        return new Executer(lb.getKey());
    } else {
        return 0;
    }
}

bool
WGlobalShipEnemyAction::Executer::execute(GObject& obj)
{
    if (GShip* sh = dynamic_cast<GShip*>(&obj)) {
        if (sh->isPlayed()) {
            if (sh->getPE() != enemy) {
                sh->setPE(enemy);
                return true;
            }
        }
    }
    return false;
}

void
WGlobalShipEnemyAction::Executer::finish(int /*obj*/, int success)
{
    messageBox(format(_("Primary Enemy changed on %d ship%!1{s%}."), success), _("Global Actions"));
}
