/**
  *  \file client/dlg-groundcombat.cc
  *  \brief Ground Combat Prediction Dialog
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <algorithm>
#include "client/dlg-groundcombat.h"
#include "ui/ui.h"
#include "ui/window.h"
#include "ui/table.h"
#include "ui/skin.h"
#include "game/host.h"
#include "game/planet.h"
#include "game/config.h"
#include "util/nls.h"
#include "util/misc.h"
#include "game/formula.h"
#include "ui/uiglobal.h"
#include "cpluslib/format.h"
#include "ui/layout.h"
#include "gfx/palette.h"
#include "game/racename.h"
#include "ui/widgets/keytranslator.h"
#include "ui/widgets/button.h"

namespace {
    class WGroundCombatWidget : public UIWidget {
        const GPlanet& planet;
        int defense;

        void drawTable(UIAbstractTableWriter& w);
     public:
        WGroundCombatWidget(const GPlanet& planet, int defense);
        ~WGroundCombatWidget();

        void drawContent(GfxCanvas& can);
        void getLayoutInfo(LayoutInfo& li);
    };
}

WGroundCombatWidget::WGroundCombatWidget(const GPlanet& planet, int defense)
    : planet(planet),
      defense(defense)
{ }

WGroundCombatWidget::~WGroundCombatWidget()
{ }

/** Display content table. Used for both sizing and drawing.
    FIXME: the original version in PCC 1.x. has the headings underlined,
    which looks much better. */
void
WGroundCombatWidget::drawTable(UIAbstractTableWriter& w)
{
    const char* defense_fmt = _("(defense) %d");
    const char* header_text = _("Clans needed to win");

    // Compute my strength
    int32 myStrength;
    if (host.isPHost()) {
        // PS := Colonists * (20 * GDEFENSE + dp) \ 20
        // i.e. for 0 dp planet = Colonists * GDEFENSE
        //         20 dp planet = Colonists * (GDEFENSE+1)
        myStrength = (planet.getColonists() *
                      (20 * config.GroundDefenseFactor(planet.getOwner()) + defense))
            / 20;
    } else {
        // PS := ERnd( Colonists * GDEFENSE * (Defense + 20) / 20 )
        // i.e. for 0 dp planet = Colonists * GDEFENSE
        //         20 dp planet = Colonists * GDEFENSE*2 }
        myStrength = rdivaddl(planet.getColonists() * config.GroundDefenseFactor(planet.getOwner()) * (defense + 20),
                              20, 0);
    }

    // Left column
    w.newLColumn();
    w.writeFixedCell(UIColor::tc_Static, _("Attacker"));
    for (int i = 1; i <= NUM_PLAYERS; ++i)
        w.writeFixedCell(i == planet.getOwner() ? UIColor::tc_Green : UIColor::tc_Static,
                         player_racenames.getFullName(i));

    // Right column
    w.newRColumn(std::max(fonts[FONT_NORMAL]->getTextWidth(format(defense_fmt, 999999)),
                          fonts[FONT_NORMAL]->getTextWidth(header_text)));
    w.writeFixedCell(UIColor::tc_Static, header_text);
    for (int i = 1; i <= NUM_PLAYERS; ++i) {
        if (i == planet.getOwner()) {
            w.writeVariableCell(UIColor::tc_Green, format(defense_fmt, planet.getColonists()));
        } else {
            // given N=number of attacking clans, their strength is
            //    N * GATTACK
            // which must be >= myStrength for the attack to kill our planet.
            //    N * GATTACK >= myStrength
            // -> N           >= myStrength / GATTACK
            // the minimum N hence is ceil(myStrength / GATTACK)
            int32 theirRatio = config.GroundKillFactor(i);
            if (theirRatio > 0) {
                int32 equivClans = (myStrength + (theirRatio-1)) / theirRatio;
                w.writeVariableCell(UIColor::tc_Static, numToString(equivClans));
            } else {
                w.writeVariableCell(UIColor::tc_Static, string_t());
            }
        }
    }

    w.finish();
}

void
WGroundCombatWidget::drawContent(GfxCanvas& can)
{
    GfxContext ctx;
    ctx.useCanvas(can).useColorScheme(getSkin());
    UITableWriter writer(ctx, getExtent(), FONT_NORMAL);
    drawTable(writer);
    can.update(getExtent());
}

void
WGroundCombatWidget::getLayoutInfo(LayoutInfo& li)
{
    UITableSizer sizer(FONT_NORMAL);
    drawTable(sizer);

    li.min_size = li.pref_size = GfxPoint(sizer.getWidth(), sizer.getHeight());
    li.flags = 0;
}

/** Display Ground Combat Prediction dialog for own planet.
    \param planet Planet to display dialog for. Must be owned by player, i.e. have full info
    \param defense Assume this many defense posts */
void
doGroundCombatPrediction(const GPlanet& planet, int defense)
{
    ObjectHolder h;

    UIWindow& win = h.add(new UIWindow(_("Ground Combat"), UIWindow::BLUE_WINDOW, COLOR_GRAY));
    win.setLayout(UIVBoxLayout::instance5);

    win.add(h.add(new WGroundCombatWidget(planet, defense)));

    UIGroup& group = h.add(new UIGroup(UIHBoxLayout::instance5));
    group.add(h.add(new UISpacer()));
    group.add(h.add(new UIButton(1, _("OK"), SDLK_RETURN)).addStop());
    group.add(h.add(new UISpacer()));
    win.add(group);
    win.add(h.add(new UIKeyTranslator())
            .add(' ', 1, true)
            .add(SDLK_ESCAPE, 1, true));

    win.pack();
    ui_root->centerWidget(win);
    ui_root->runModal(win);
}
