/**
  *  \file client/screens/histscreen.cc
  *  \brief Ship History Screen (F6)
  *
  *  Unlike the regular control screens, this displays not just an
  *  object, but allows users to move through time. WControlScreen
  *  just supports distinction between objects.
  *
  *  Therefore, this wraps the GHistoryShipType within a
  *  WHistoryObjectType, which also supports tracking a turn number. 
  *  Tiles can detect the type to support movement through time:
  *  - WHistoryShipPositionTile sets the turn number according to
  *    its selection
  *  - other tiles (WHistoryShipMovementTile) and WHistoryScreen
  *    itself may query it as needed.
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/screens/histscreen.h"
#include "client/cscreen.h"
#include "client/screens.h"
#include "client/tiles.h"
#include "game/global.h"
#include "game/ship.h"
#include "int/if/shipif.h"
#include "int/intglobal.h"
#include "int/keymap.h"
#include "client/wscanchart.h"

namespace {
    /** History screen. */
    class WHistoryScreen : public WControlScreen {
        WHistoryObjectType type;
     public:
        WHistoryScreen();
        ~WHistoryScreen();
        void update();
        void setScriptContext();
        void onTurnChange();
        bool handleEvent(const UIEvent& event, bool second_pass);
    };

    /** Scanner chart for history screen. Displays the chosen ship's track. */
    class WHistoryScannerChart : public WScannerChartWidget {
        GObjectType& type;
     public:
        WHistoryScannerChart(GObjectType& type);
        void drawPre(GfxCanvas& can);
    };
}

/*************************** WHistoryObjectType **************************/

WHistoryObjectType::WHistoryObjectType(GObjectType& base)
    : GAliasObjectType(base),
      turn(0)
{ }

/** Set turn number. Raises sig_turn_changed if this is a change. */
void
WHistoryObjectType::setTurn(int turn)
{
    if (this->turn != turn) {
        this->turn = turn;
        sig_turn_changed.raise();
    }
}

/** Get turn number. */
int
WHistoryObjectType::getTurn() const
{
    return turn;
}

/** Browse through ships at current position. */
void
WHistoryObjectType::browse(bool backward, bool marked_only)
{
    /* Get initial state. Exit when we have no position. */
    int start = getCurrent();
    GPoint pt = getPos();
    if (pt.x < 0) {
        return;
    }

    /* Browse through ships */
    int id = getCurrent();
    while (1) {
        id = backward ? getPreviousWrap(id, marked_only) : getNextWrap(id, marked_only);
        if (id == 0 || id == start) {
            /* No object, or one loop completed */
            return;
        }

        /* Check whether that ship ever was at the requested location */
        if (const GShip* sh = dynamic_cast<GShip*>(&getObject(id))) {
            int turn = 0;
            for (int i = sh->getHistoryNewestTurn(), e = sh->getHistoryOldestTurn(); i > e; --i) {
                const TDbShipTrackEntry& track = sh->getHistoryEntry(i);
                if (track.x == pt.x && track.y == pt.y) {
                    turn = i;
                    break;
                }
            }

            if (turn != 0) {
                /* Found it! */
                setCurrent(id);
                setTurn(turn);
                return;
            }
        }
    }
}

/** Get current position. When this type sits at a ship with a valid
    turn, returns the ship's position in that turn. Otherwise, returns
    GPoint(-1,-1) denoting an invalid position. */
GPoint
WHistoryObjectType::getPos()
{
    if (getUniverse() != 0) {
        if (GShip* ship = dynamic_cast<GShip*>(&getObject(getCurrent()))) {
            const TDbShipTrackEntry& track = ship->getHistoryEntry(turn);
            if (track.x >= 0 && track.y >= 0) {
                return GPoint(track.x, track.y);
            }
        }
    }
    return GPoint(-1, -1);
}

/************************** WHistoryScannerChart *************************/

WHistoryScannerChart::WHistoryScannerChart(GObjectType& type)
    : type(type)
{ }

void
WHistoryScannerChart::drawPre(GfxCanvas& can)
{
    /* WScannerChartWidget has prepared the chart viewport. Modify it
       to include the chosen ship's track, if we have one. */
    int vec_id = -1;
    if (type.getUniverse() != 0) {
        if (GShip* sh = dynamic_cast<GShip*>(&type.getObject(type.getCurrent()))) {
            vec_id = sh->getId();
        }
    }
    getViewport().vec_id = vec_id;
    WScannerChartWidget::drawPre(can);
}

/***************************** WHistoryScreen ****************************/

/** Constructor.

    Implementation note: the cleanest solution would have been to let
    WControlScreen observe /type/. However, C++ makes it difficult to
    pass a member to a base class destructor. Since /type/ does not
    change /global_histship_type/'s object set, giving WControlScreen
    /global_histship_type/ works, too. We just have to keep the cursors
    in sync. This would have been a good idea in any case, because it
    allows external code (scripts!) to query the current ship on the
    history screen.

    \todo Currently this just synchronizes cursors in one direction,
    so using a WObjectTypeHeaderTile on a history screen will not work. */
WHistoryScreen::WHistoryScreen()
    : WControlScreen(global_histship_type, WScreenId::HistoryScreen),
      type(global_histship_type)
{
    createTileLayout(*this, "HISTORYSCREEN", type);
    addChart(holder.add(new WHistoryScannerChart(type)));
    setKeymap(IntKeymap::getKeymapByName("HISTORYSCREEN"));

    /* Observe turn changes. WControlScreen will observe object
       changes on /global_histship_type/, which will mirror /type/, so
       we don't have to observe that, too. */
    type.sig_turn_changed.add(this, &WHistoryScreen::onTurnChange);
}

WHistoryScreen::~WHistoryScreen()
{ }

void
WHistoryScreen::update()
{
    /* This is called upon object changes */
    type.setCurrent(global_histship_type.getCurrent());
    onTurnChange();
}

void
WHistoryScreen::setScriptContext()
{
    global_contexts.push_back(new IntShipContext(getType().getCurrent()));
}

void
WHistoryScreen::onTurnChange()
{
    /* This is called upon turn changes */
    GPoint pt = type.getPos();
    if (pt.x >= 0) {
        getScanner().getChart()->setCenterPosition(pt);
    } else {
        getScanner().getChart()->setInvalid();
    }
}

bool
WHistoryScreen::handleEvent(const UIEvent& event, bool second_pass)
{
    if (event.type == ev_Key && (event.key.code & ~(ss_Ctrl + ss_Shift)) == SDLK_TAB) {
        /* Cycle through ships here */
        type.browse((event.key.code & ss_Shift) != 0, (event.key.code & ss_Ctrl) != 0);
        return true;
    }
    return WControlScreen::handleEvent(event, second_pass);
}

void
doHistoryScreen()
{
    WHistoryScreen().runScreen();
}
