/**
  *  \file client/tiles/mineinfo.cc
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/mineinfo.h"
#include "game/global.h"
#include "game/mine.h"
#include "game/coord.h"
#include "game/planet.h"
#include "ui/uiglobal.h"
#include "util/nls.h"
#include "cpluslib/format.h"
#include "game/team.h"
#include "game/racename.h"
#include "game/format.h"

/** Find planet associated with a minefield.
    \return planet Id, or 0 */
static int
getAssociatedPlanet(const GGameTurn& turn, const GMinefield& mf)
{
    // chartdlg.pas, CMinefieldView.SetMinefield

    int pid  = 0;
    long dist = 0;
    for (int i = 1; i <= NUM_PLANETS; ++i) {
        const GPlanet& p = turn.getCurrentUniverse().getPlanet(i);
        if (p.isExistant()) {
            /* The planet is a possible controlling planet if...
               - we know it has the same owner as the minefield
               - we don't know the planet's owner, and don't have
                 full data for the minefield owner's race */
            bool maybe = (p.getOwner() == mf.getOwner())
                || (!turn.isData(mf.getOwner()) && !p.isOwnerKnown());

            /* Choose closest planet */
            if (maybe) {
                long ndist = distanceSquared(p.getPos(), mf.getPos());
                if (pid == 0 || ndist < dist) {
                    pid  = i;
                    dist = ndist;
                }
            }
        }
    }

    return pid;
}

/*************************** WMinefieldInfoTile **************************/

WMinefieldInfoTile::WMinefieldInfoTile(GObjectType& type)
    : WObjectTypeObserverTile(font_sizes[FONT_NORMAL].scaledBy(28, 7), type),
      btn_s("S", 's')
{
    const int em = font_heights[FONT_NORMAL];
    setLayout(RightJustified, 0, false);
    btn_s.setState(UIButton::bf_SmallSize, true);
    addControl(GfxRect(0, 0, em-1, em-1), btn_s);
}

void
WMinefieldInfoTile::drawData(GfxCanvas& can)
{
    // |----12em-----|------16em----------------|
    // Owner:         Fed
    // Size:          32 ly radius
    //                999 units
    // After decay:   900 units (30 ly)
    // Last info:     current turn
    // Controlled by: Wolf 359
    //                Fed planet

    GfxRect pos  = getInnerSize(0, 0);
    const int dy = font_heights[FONT_NORMAL];
    const int x0 = 0;
    const int x1 = font_ems[FONT_NORMAL]*12;
    const int w  = pos.w - x1;
    int y = 0;

    GMinefield* mf = dynamic_cast<GMinefield*>(getObject());
    if (!mf)
        return;

    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    ctx.useFont(*fonts[FONT_NORMAL]);
    ctx.setColor(UIColor::tc_Static);
    ctx.setSolidBackground();

    // Line 1:
    outText (ctx, x0, y, _("Owner:"));
    outTextF(ctx, x1, y, w, player_racenames.getShortName(mf->getOwner()));
    y += dy;

    // Line 2+3:
    outText (ctx, x0, y, _("Size:"));
    outTextF(ctx, x1, y, w, format(_("%d ly radius"), mf->getRadius()));
    y += dy;
    outTextF(ctx, x1, y, w, format(_("%d units"), mf->getUnits()));
    y += dy;

    // Line 4:
    int32 uad = mf->getUnitsAfterDecay(mf->getUnits());
    outText (ctx, x0, y, _("After decay:"));
    outTextF(ctx, x1, y, w, format(_("%d units (%d ly)"), uad, mf->getRadiusFromUnits(uad)));
    y += dy;

    // Line 5:
    int age = getDisplayedTurn().getTurnNumber() - mf->getTurnLastSeen();
    outText (ctx, x0, y, _("Last info:"));
    outTextF(ctx, x1, y, w, formatTurnNumber(mf->getTurnLastSeen()));
    y += dy;

    // Lines 6+7:
    int pid = getAssociatedPlanet(getDisplayedTurn(), *mf);
    outText (ctx, x0, y, _("Controlled by:"));
    if (pid == 0) {
        outTextF(ctx, x1, y, w, _("unknown"));
        y += dy;
        outTextF(ctx, x1, y, w, string_t());
    } else {
        const GPlanet& p = getDisplayedTurn().getCurrentUniverse().getPlanet(pid);
        outTextF(ctx, x1, y, w, p.getName());
        y += dy;
        if (!p.isOwnerKnown())
            outTextF(ctx, x1, y, w, _("a planet with unknown owner"));
        else if (p.getOwner() == 0)
            outTextF(ctx, x1, y, w, _("unowned planet"));
        else if (p.getOwner() == getPlayerId())
            outTextF(ctx, x1, y, w, _("our planet"));
        else
            outTextF(ctx, x1, y, w, player_racenames.getShortName(p.getOwner()));
    }
    y += dy;
}

