/**
  *  \file util/recode.h
  *  \brief Character-set Recoding
  *
  *  Declarations of the interfaces used for character set
  *  translation.
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UTIL_RECODE_H
#define PCC_V2_UTIL_RECODE_H

#include <cstddef>
#include "cpluslib/string.h"
#include "defs.h"
#include "cpluslib/types.h"

/*! \class CharacterSet
    \brief Character Set Handle

    A CharacterSet object is a handle to a character set. You can use
    CharacterSet::getCharsetByName() to obtain a CharacterSet object
    for a character set given by its name. */
class CharacterSet {
 public:
    class Data;
 private:
    const Data* data;
 public:
    /** Create character set with given data.
        \param data character set definition. 0 defines an untranslated character set. */
    CharacterSet(const Data* data = 0) : data(data)
        { }

    const char* getName() const;
    const char* getDescription() const;
    uint16 getUnicode(char c) const;
    char getCharacter(uint16 uni) const;

    string_t convertToUtf8(string_t str) const;
    string_t convertFromUtf8(string_t str) const;

    /** Check whether this is a valid character set.
        \return true iff character set is valid, translation is known.
        false if the character set is unknown and we fall back to untranslated
        mode */
    bool isValid() const
        { return data != 0; }

    /** Compare for equality. */
    bool operator==(CharacterSet rhs) const
        { return data==rhs.data; }

    /** Compare for inequality. */
    bool operator!=(CharacterSet rhs) const
        { return data!=rhs.data; }

    static CharacterSet  getCharsetByName(string_t name);
    static bool          isCharsetNameUtf8(string_t name);
    static std::size_t   getNumCharsets();
    static CharacterSet  getCharset(std::size_t n);
};

void setGameCharacterSet(CharacterSet gcs);
CharacterSet getGameCharacterSet();
bool isReliableGameCharacter(uint32_t ch);
string_t convertUtf8ToGame(string_t str);
string_t convertGameToUtf8(string_t str);

#endif
